<?xml version="1.0" encoding="ISO-8859-1"?>
<!--*****************************************************************************************************************
Written by Giles Hogben,
Joint Research Center, Ispra Italy.
Note this Stylesheet uses 
****************************************************************************************************************-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:msxsl="urn:schemas-microsoft-com:xslt" xmlns:xsd="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="msxsl xsd xsl">
	<xsl:key name="DATADEFs" match="DATA-DEF" use="substring-before(@name,'.')"/>
	<xsl:key name="Categories" match="//CATEGORIES/*" use="local-name()"/>
	<!--***************************************************************************************************************** 
Multipass template (runs the document through pass1 and then pass2 templates.

NB: the multipass technique relies on an xslt node-set extension. This sheet uses the MSXSL extension.
To use SAXON, just change line 42 "msxsl:node-set...." to "xx:node-set" and insert the namespace in the stylesheet declaration.

@br/~ are inserted for later replacement by line breaks. For output not as HTML, just remove.
****************************************************************************************************************-->
	<xsl:template match="/">
		<xsl:variable name="temp">
			<xsl:apply-templates select="/" mode="pass1"/>
		</xsl:variable>
		<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema">
			<xsd:element name="data-group">
				<xsd:complexType>
					<xsd:sequence>
						<xsd:element ref="datatype" minOccurs="0" maxOccurs="unbounded"/>
					</xsd:sequence>
				</xsd:complexType>
			</xsd:element>
			<!--*************************Create Overall Categories Data Type************************************************-->
			<xsl:value-of select="count(//*[generate-id(.)=generate-id(key('Categories',local-name()))])"/>
			<xsd:simpleType name="allCategories">
				<xsd:restriction base="xsd:string">
					<xsl:for-each select="//*[generate-id(.)=generate-id(key('Categories',local-name()))]">
			 			@br/~<xsd:enumeration>
							<xsl:attribute name="value"><xsl:value-of select="local-name()"/></xsl:attribute>
						@br/~</xsd:enumeration>
					</xsl:for-each>
				</xsd:restriction>
			</xsd:simpleType>
		</xsd:schema>
	</xsl:template>
	
	<!--*****************************************************************************************************************
This template does most of the processing.
****************************************************************************************************************-->
	<xsl:template match="/" mode="pass1">
		<!--************** Output initial list of root data types inside datatype element ***************************-->
	@br/~<xsd:element name="datatype">@br/~
<!--********************Group of root data classes ****************************************************************-->
			<xsd:complexType>
				<xsd:sequence>
					<xsl:for-each select="//DATA-DEF[generate-id(.)=generate-id(key('DATADEFs', substring-before(@name,'.')))]">
						<xsd:element minOccurs="0" maxOccurs="1">
							<xsl:attribute name="ref"><xsl:value-of select="substring-before(@name,'.')"/></xsl:attribute>
						@br/~</xsd:element>	@br/~
					</xsl:for-each>
				</xsd:sequence>
			</xsd:complexType>
		</xsd:element>@br/~

<!--*****************************************************************************************************************
For each root data type, output the allowed subtypes and recurse
First Iterate through the key (hash table) of root elements 
****************************************************************************************************************-->
		<xsl:for-each select="//DATA-DEF[generate-id(.)=generate-id(key('DATADEFs', substring-before(@name,'.')))]">
			<xsd:element>
				<xsl:attribute name="name"><xsl:value-of select="substring-before(@name,'.')"/></xsl:attribute>
				<!--********************Sub-elements must be either categories or data types but not both. There may only be one of the subelements allowed-->
				<xsd:complexType mixed="true">
					<xsd:choice>
						<!--******************* Next level of subelements *************************************************************-->
						<xsl:for-each select="key('DATADEFs', substring-before(@name,'.'))">
							<xsl:call-template name="nextLevel"/>
						</xsl:for-each>
					</xsd:choice>
				</xsd:complexType>
			</xsd:element>@br/~
<!--****************** Now recursively iterate for all further subtypes    *************************************-->
			<xsl:call-template name="outerlooper">
				<xsl:with-param name="i">0</xsl:with-param>
				<xsl:with-param name="linkTo" select="substring-before(@name,'.')"/>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>
	<!--*****************************************************************************************************************
This template controls the inner iteration Goes deeper and deeper into the tree outputting nodes, always starting from the root 
****************************************************************************************************************-->
	<xsl:template name="outerlooper">
		<xsl:param name="i" select="/.."/>
		<xsl:param name="linkTo" select="/.."/>
		<xsl:choose>
			<xsl:when test="$linkTo!=''">
				<!--***********************If there is a structref, drill into the tree recursively, if not, output the value**-->
				<xsl:variable name="EmptyTest">
					<xsl:call-template name="innerlooper">
						<xsl:with-param name="outerloop" select="$i"/>
						<xsl:with-param name="innerloop" select="0"/>
						<xsl:with-param name="linkTo" select="$linkTo"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:copy-of select="$EmptyTest"/>
				<!--***********************Termination condition and recurse **************************************************-->
				<xsl:if test="string(normalize-space($EmptyTest))!=string('') and $i&lt;5">
					<xsl:call-template name="outerlooper">
						<xsl:with-param name="i">
							<xsl:value-of select="$i+1"/>
						</xsl:with-param>
						<xsl:with-param name="linkTo" select="$linkTo"/>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<!--*************************If it is data-def only************************************************************-->
			<xsl:otherwise>
				<xsl:for-each select="//DATA-DEF[generate-id(.)=generate-id(key('DATADEFs', substring-before(@name,'.')))]">
					<xsl:for-each select="key('DATADEFs', substring-before(@name,'.'))">
					@br/~<xsd:element>
							<xsl:attribute name="name"><xsl:value-of select="substring-after(@name,'.')"/></xsl:attribute>
								<!--***********************Add Short and long descriptions as annotations**************************************-->
								<xsl:call-template name="annotations"/>
								<!--***********************End add Short and long descriptions as annotations**************************************-->
								<!--*************************Categories (as many as you like from the allowed list although repeated elements don't add anything)-->
								<xsd:complexType mixed="true">
									<xsl:call-template name="getCategories"/>
								</xsd:complexType>
								<!--*************************End of Categories at this level ***************************************************-->
					@br/~</xsd:element>
					</xsl:for-each>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--*****************************************************************************************************************
Template to Recurse down into the tree to iterative depth  
****************************************************************************************************************-->
	<xsl:template name="innerlooper">
		<xsl:param name="outerloop" select="/.."/>
		<xsl:param name="innerloop" select="/.."/>
		<xsl:param name="linkTo" select="/.."/>
		<xsl:for-each select="//*[substring-before(@name,'.')=$linkTo]">
			<xsl:choose>
				<xsl:when test="$innerloop=$outerloop">
					<xsd:element>
						<xsl:attribute name="name"><xsl:value-of select="substring-after(@name,'.')"/></xsl:attribute>
						<!--***********************Add Short and long descriptions as annotations**************************************-->
						<xsl:call-template name="annotations"/>
						<!--***********************End add Short and long descriptions as annotations**************************************-->
						<xsl:variable name="tempStructRef2" select="substring-after(@structref,'#')"/>
						<xsl:choose>
							<xsl:when test="count(//DATA-STRUCT[substring-before(@name,'.')=$tempStructRef2])!=0">
								<xsd:complexType>
									<xsd:choice>
										<xsl:for-each select="//DATA-STRUCT[substring-before(@name,'.')=$tempStructRef2]">
											<!--
										*****Iteration Number***********
										<xsl:value-of  select="$outerloop+3"/> 
										*****Iteration Number***********
									-->
											<xsl:call-template name="nextLevel"/>
										</xsl:for-each>
									</xsd:choice>
								</xsd:complexType>
							</xsl:when>
							<xsl:otherwise>
							<xsd:complexType mixed="true">
								<xsl:call-template name="getCategories"/>
							</xsd:complexType>
							</xsl:otherwise>
						</xsl:choose>
					</xsd:element>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="string(substring-after(@structref,'#'))"/>
					<xsl:choose>
						<xsl:when test="string(substring-after(@structref,'#'))!=string('')">
							<xsl:call-template name="innerlooper">
								<xsl:with-param name="innerloop" select="$innerloop+1"/>
								<xsl:with-param name="outerloop" select="$outerloop"/>
								<xsl:with-param name="linkTo" select="substring-after(@structref,'#')"/>
							</xsl:call-template>
						</xsl:when>
						<xsl:otherwise>
							<xsl:call-template name="finalLevel"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>

	<!--*****************************************************************************************************************
Template to get categories at a particular level
****************************************************************************************************************-->
	<xsl:template name="getCategories">
		<xsl:choose>
			<xsl:when test="count(./CATEGORIES/*) &gt; 0">
				<xsd:sequence>
					<xsd:element name="CATEGORY" minOccurs="0" maxOccurs="unbounded">
						<xsd:simpleType>
							<xsd:restriction base="allCategories">
								<xsd:pattern>
									<xsl:attribute name="value"><xsl:for-each select="./CATEGORIES/*"><xsl:if test="position()!=1">|</xsl:if><xsl:value-of select="local-name()"/></xsl:for-each></xsl:attribute>
								</xsd:pattern>
							</xsd:restriction>
						</xsd:simpleType>
					</xsd:element>
				</xsd:sequence>
			</xsl:when>
			<!-- Variable category element -->
			<xsl:otherwise>
				<xsd:sequence>
				<xsd:element name="CATEGORY" minOccurs="0" maxOccurs="unbounded" type="allCategories"/>
				</xsd:sequence>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--*****************************************************************************************************************
Template to output an element with a ref attribute
****************************************************************************************************************-->
	<xsl:template name="getNextElementRef">
	@br/~<xsd:element minOccurs="0" maxOccurs="1">
			<xsl:attribute name="ref"><xsl:value-of select="substring-after(@name,'.')"/></xsl:attribute>
	@br/~</xsd:element>
	</xsl:template>
	<!--*****************************************************************************************************************
Template outputs the next level (used in recursive template)
****************************************************************************************************************-->
	<xsl:template name="nextLevel">
			@br/~<xsd:choice>
			<!--******************EITHER the next element down ************************************************************-->
			<xsl:call-template name="getNextElementRef"/>
			<!--*****************OR Categories (as many as you like from the allowed list although repeated elements don't add anything)-->
			<xsl:call-template name="getCategories"/>
			<!--**************** End of Categories at this level ***********************************************************-->
			@br/~</xsd:choice>
	</xsl:template>
	<!--*****************************************************************************************************************
Template to output final level in a recursion (with a name element rather than a ref element)
****************************************************************************************************************-->
	<xsl:template name="finalLevel">
		<!--******************EITHER the next element down ************************************************************-->
		<xsl:call-template name="getFinalElementName"/>
		<!--*****************OR Categories (as many as you like from the allowed list although repeated elements don't add anything)-->
	</xsl:template>
	<!--*****************************************************************************************************************
Template to output a name element alone.
****************************************************************************************************************-->
	<xsl:template name="getFinalElementName">
		<xsd:element>
			<xsl:attribute name="name"><xsl:value-of select="substring-after(@name,'.')"/></xsl:attribute>
				<xsl:call-template name="annotations"/>
			<xsd:complexType mixed="true">
				<xsl:call-template name="getCategories"/>
			</xsd:complexType>
		</xsd:element>
	</xsl:template>
	<!--*******************************************************************************************************************
Template to add annotations
***********************************************************************************************************************-->
	<xsl:template name="annotations">
		<xsl:if test="normalize-space(@short-description)!=''">
			<xsd:annotation>
				<xsd:documentation>
					<xsl:value-of select="@short-description"/>
				</xsd:documentation>
			</xsd:annotation>
		</xsl:if>
		<xsl:if test="normalize-space(.)!=''">
			<xsd:annotation>
				<xsd:documentation>
					<xsl:value-of select="."/>
				</xsd:documentation>
			</xsd:annotation>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
